//Bibliothek zur kommunikation mit einem WiFi Netzwerk
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>

//Bibliothek zum auslesen der Sensorwerte
//vom SHT30 Sensor
#include <WEMOS_SHT3X.h>
//instanziieren eines Objektes vom Typ SHT3X
//mit der I2C Adresse 0x44
SHT3X sht30(0x44);

//Zugangsdaten zum lokalen WiFi-Netzwerk
const char* ssid = "";
const char* password = "";

//Adresse des Webhooks
String webhook = "<<Adresse aus der Dokumentation zu den Webhooks>>";

//Felder für die Temperatur & die rel. Luftfeuchtigkeit
float temperature = 0;
float humidity = 0;

//Konstante für die maximale Temperatur,
//wenn dieser Wert überschritten wird,
//dann soll eine Benachrichtigung gesendet werden
const float MAX_TEMP = 18.0;

//Intervall zum lesen & auswerten des Sensors
int PAUSE = 5000;
long lastUpdate = 0;

//Intervall zum senden einer Benachrichtigung
//Der Sensor wird zunächst alle 5 Sekunden gelesen
//eine Benachrichtigung soll jedoch maximal
//jede Minute gesendet werden
int PAUSE_SEND_NOTIFICATION = 60000;
long lastSendNotification = 0;

void setup() {
  //beginn der seriellen Kommunikation mit 9600 baud
  Serial.begin(9600);

  WiFi.begin(ssid, password);
  Serial.println("Aufbau der WiFi Verbindung zu " + ssid);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("Verbindung erfolgreich aufgebaut!");
  Serial.print("IP-Adresse lautet: ");
  Serial.println(WiFi.localIP());
}

/**
* Funktion zum auslesen der Temperatur und rel. Luftfeuchtigkeit
* vom SHT30 Sensor.
**/
void readSensorValues() {
  //Wenn die kommunikation per I2C mit dem Sensor
  //erfolgreich war, dann...
  if (sht30.get() == 0) {
    //lesen der Temperatur in Grad Celsius
    temperature = sht30.cTemp;
    //lesen der rel. Luftfeuchtigkeit in %
    humidity = sht30.humidity;
    //ausgeben der Temperatur
    Serial.print("Temperatur: ");
    Serial.print(temperature);
    Serial.println("°C");
    //ausgeben der rel. Luftfeuchtigkeit
    Serial.print("rel. Luftfeuchtigkeit: ");
    Serial.print(humidity);
    Serial.println("%");
  }
}

/**
* Funktion zum absenden einer Benachrichtigung.
**/
void sendNotification() {
  //Wenn eine Verbindung zu einem WiFi-Netzwerk besteht, dann...
  if (WiFi.status() == WL_CONNECTED) {
    WiFiClient client;
    HTTPClient http;
    //absenden des Request an IFTTT
    http.begin(client, webhook.c_str());
    //lesen der Antwort
    //HTTP200 - OK
    //> HTTP200 dann fehler
    int httpResponseCode = http.GET();

    Serial.print("HTTP Status Code: ");
    Serial.println(httpResponseCode);
    String payload = http.getString();
    Serial.println(payload);

    http.end();
  }
}

void loop() {
  //Wenn der letzte Zeitpunkt plus der Pause kleiner als 
  //die Vergangenen Millisekunden ist, dann...
  if ((lastUpdate + PAUSE) < millis()) {
    //lesen der Sensorwerte
    readSensorValues();
    //überschreiben des Zeitstempels mit den aktuellen Millisekunden
    lastUpdate = millis();
    //Wenn die aktuelle Temperatur größer als der Wert der Konstante MAX_TEMP ist UND
    //der Zeitpunkt des letzten absendens einer Benachrichtung plus der Pause kleiner als die aktuellen
    //Millisekunden ist, dann...
    if ((temperature > MAX_TEMP) && (lastSendNotification + PAUSE_SEND_NOTIFICATION) < millis()) {
      //überschreiben des Feldes lastSendNoticifaction mit den aktuellen Millisekunden
      lastSendNotification = millis();
      //absenden einer Benachrichtigung
      sendNotification();
    }
  }
}
